@@ -218,7 +218,10 @@ t_cached_object *search_cache(t_session *session, char *file) {
 	return result;
 }
 
-void done_with_cached_object(t_cached_object *object) {
+void done_with_cached_object(t_cached_object *object, bool remove_object) {
+	if (remove_object) {
+		object->timer = 0;
+	}
 	(object->in_use)--;
 }
 
