@@ -225,7 +225,7 @@ pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 		case -1:
 			break;
 		case 0:
-			/* Child, executes CGI program.
+			/* Child. Executes CGI program.
 			 */
 			dup2(post_pipe[0], STDIN_FILENO);
 			dup2(html_pipe[1], STDOUT_FILENO);
@@ -285,9 +285,10 @@ pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 			run[i] = NULL;
 
 			execvp(run[0], run + 1);
+			perror("execute CGI");
 			exit(EXIT_FAILURE);
 		default:
-			/* Parent, reads CGI output
+			/* Parent. Reads CGI output.
 			 */
 			close(post_pipe[0]);
 			close(error_pipe[1]);
