@@ -7,13 +7,11 @@
 #include <dirent.h>
 #include <pthread.h>
 #include <sys/wait.h>
+#include "global.h"
 #include "client.h"
 #include "command.h"
 #include "log.h"
 
-#define ip_index(ip) (ip & 255)
-#define TIMER_OFF -1
-
 typedef struct type_client {
 	t_session *session;
 	int delayed_remove;
@@ -22,9 +20,9 @@ typedef struct type_client {
 } t_client;
 
 typedef struct type_banned {
-	long ip;
-	int  timer;
-	int  bantime;
+	t_ipv4 ip;
+	int    timer;
+	int    bantime;
 	unsigned long connect_attempts;
 	
 	struct type_banned *next;
@@ -184,7 +182,7 @@ int remove_client(t_session *session, bool free_session) {
 
 /* Check whether to allow or deny a new connection.
  */
-int connection_allowed(long ip, int maxperip, int maxtotal) {
+int connection_allowed(t_ipv4 ip, int maxperip, int maxtotal) {
 	bool banned = false;
 	t_banned *ban;
 	int perip = 0, total = 0, i;
@@ -273,7 +271,7 @@ int disconnect_clients(t_config *config) {
 
 /* Kick an IP address.
  */
-int kick_ip(long ip) {
+int kick_ip(t_ipv4 ip) {
 	t_client *client;
 	int result = 0;
 
@@ -330,7 +328,7 @@ int kick_client(int id) {
 
 /* IP ban functions
  */
-int ban_ip(long ip, int timer, bool kick_on_ban) {
+int ban_ip(t_ipv4 ip, int timer, bool kick_on_ban) {
 	int retval = 0;
 	t_banned *ban;
 	bool new_ip = true;
@@ -378,7 +376,7 @@ int ban_ip(long ip, int timer, bool kick_on_ban) {
 
 /* Reset the timer of a banned IP address.
  */
-int reban_ip(long ip) {
+int reban_ip(t_ipv4 ip) {
 	t_banned *ban;
 	int result = 0;
 
@@ -434,7 +432,7 @@ void check_ban_list(t_config *config) {
 
 /* Unban an IP address.
  */
-int unban_ip(long ip) {
+int unban_ip(t_ipv4 ip) {
 	t_banned *ban, *prev = NULL, *next;
 	int result = 0;
 
