@@ -79,6 +79,7 @@ char *hs_conlen      = "Content-Length: ";
 char *fb_filesystem  = "access denied via filesystem";
 char *fb_symlink     = "symlink not allowed";
 char *fb_accesslist  = "access denied via accesslist";
+char *fb_alterlist   = "access denied via alterlist";
 char *version_string = "Hiawatha v"VERSION
 #ifdef HAVE_SSL
 			", SSL"
@@ -224,9 +225,17 @@ int handle_request(t_session *session) {
 			retval = 501;
 		}
 	} else if (strcmp(session->method, "PUT") == 0) {
-		retval = 501;
+		if (session->binding->enable_alter) {
+			retval = handle_put_request(session);
+		} else {
+			retval = 501;
+		}
 	} else if (strcmp(session->method, "DELETE") == 0) {
-		retval = 501;
+		if (session->binding->enable_alter) {
+			retval = handle_delete_request(session);
+		} else {
+			retval = 501;
+		}
 	} else if (strcmp(session->method, "CONNECT") == 0) {
 		retval = 501;
 	} else if (strcmp(session->method, "PROPFIND") == 0) {
@@ -278,6 +287,23 @@ int fetch_request(t_session *session) {
 							}
 							session->content_length = content_length;
 							*strend = '\r';
+
+							if (header_length + content_length > session->binding->max_request_size) {
+								keep_reading = false;
+								result = fe_MAX_REQUESTSIZE;
+								break;
+							}
+
+							if (header_length + content_length + 1 > session->buffer_size) {
+								session->buffer_size = header_length + content_length + 1;
+								if ((new_reqbuf = (char*)realloc(session->request, session->buffer_size)) != NULL) {
+									session->request = new_reqbuf;
+								} else {
+									result = fe_READ_ERROR;
+									keep_reading = false;
+									break;
+								}
+							}
 						}
 					} else {
 						break;
@@ -313,7 +339,7 @@ int fetch_request(t_session *session) {
 				}
 				break;
 			default:
-				if ((session->buffer_size - session->bytes_in_buffer) < 256) {
+				if ((content_length == -1) && ((session->buffer_size - session->bytes_in_buffer) < 256)) {
 					session->buffer_size += REQUEST_BUFFER_CHUNK;
 					if ((new_reqbuf = (char*)realloc(session->request, session->buffer_size)) != NULL) {
 						session->request = new_reqbuf;
@@ -577,6 +603,7 @@ int serve_client(t_session *session) {
 	int result, length;
 	char *search, *hostname, *conffile;
 	t_host *host_record;
+	bool put_method;
 
 	if ((result = fetch_request(session)) != 200) {
 		return result;
@@ -584,10 +611,7 @@ int serve_client(t_session *session) {
 		return result;
 	}
 
-	parse_special_chars(session->uri, strlen(session->uri));
-	if (session->vars != NULL) {
-		parse_special_chars(session->vars, strlen(session->vars));
-	}
+	put_method = (strcmp(session->method, "PUT") == 0);
 
 	session->host = session->config->first_host;
 	if ((hostname = get_headerfield("Host:", session->headerfields)) != NULL) {
@@ -596,6 +620,11 @@ int serve_client(t_session *session) {
 		}
 	}
 
+	url_decode(session->uri);
+	if ((session->vars != NULL) && (session->host->secure_url)) {
+		remove_dangerous_chars(session->vars);
+	}
+
 	if (session->host->require_resolve_ip && (session->ip_resolved == false)) {
 		/* See if the client IP-address is resolvable
 		 */
@@ -607,6 +636,12 @@ int serve_client(t_session *session) {
 	}
 
 	if (valid_uri(session->uri) == false) {
+		if (put_method) {
+			if (session->binding->enable_alter) {
+				return 403;
+			}
+			return 501;
+		}
 		return 404;
 	} else if ((result = uri_to_path(session)) != 200) {
 		return result;
@@ -622,7 +657,7 @@ int serve_client(t_session *session) {
 				memcpy(conffile, session->file_on_disk, length);
 				memcpy(conffile + length, ".hiawatha\0", 10);
 				if (duplicate_host(session)) {
-					if (read_user_configfile(conffile, session->host) > 0) {
+					if (read_user_configfile(conffile, session->host, &(session->tempdata)) > 0) {
 						log_file_error(session, conffile, "error in configuration file");
 						result = 500;
 					}
@@ -676,7 +711,7 @@ int serve_client(t_session *session) {
 					}
 				}
 			}
-			if (session->accept_gzip == false) {
+			if ((session->accept_gzip == false) && (put_method == false)) {
 				return 404;
 			}
 	}
@@ -685,7 +720,6 @@ int serve_client(t_session *session) {
 		case deny:
 			log_error(session, fb_accesslist);
 			return 403;
-			break;
 		case allow:
 			break;
 		case pwd:
@@ -721,7 +755,7 @@ void handle_request_result(t_session *session, int result) {
 	switch (result) {
 		case fe_MAX_REQUESTSIZE:
 			log_garbage(session);
-			send_code(session, 400);
+			send_code(session, 413);
 			if ((session->config->ban_on_max_request_size > 0) && ip_allowed(session->ip_address, session->config->banlist_mask)) {
 				ban_ip(session->ip_address, session->config->ban_on_max_request_size, session->config->kick_on_ban);
 				log_system(session, "Client banned because of sending a too large request");
@@ -770,6 +804,12 @@ void handle_request_result(t_session *session, int result) {
 			break;
 		case 200:
 			break;
+		case 204:
+			if (session->data_sent == false) {
+				send_header(session, 204);
+				send_buffer(session, "\r\n", 2);
+			}
+			break;
 		case 304:
 		case 412:
 		case 413:
@@ -846,6 +886,7 @@ void connection_handler(t_session *session) {
 			}
 		} while (session->keep_alive);
 
+		destroy_session(session);
 		close_socket(session);
 	} else {
 		close(session->client_socket);
@@ -1098,6 +1139,7 @@ int run_server(t_settings *settings) {
 	/* Bind Serverports
 	 */
 	if (bind_sockets(config->binding) == -1) {
+		perror("bind http");
 		return EXIT_FAILURE;
 	}
 
@@ -1112,6 +1154,7 @@ int run_server(t_settings *settings) {
 	while (binding != NULL) {
 		if (binding->use_ssl) {
 			if ((binding->ssl_context = ssl_binding(binding->server_key, binding->ca_cert, binding->verify_depth, config->dh_file, config->allowed_ciphers)) == NULL) {
+				perror("bind https");
 				return EXIT_FAILURE;
 			}
 		}
@@ -1123,6 +1166,7 @@ int run_server(t_settings *settings) {
 	/* Bind CommandChannels
 	 */
 	if (bind_sockets(config->command_port) == -1) {
+		perror("bind command");
 		return EXIT_FAILURE;
 	}
 #endif
@@ -1198,7 +1242,7 @@ int run_server(t_settings *settings) {
 	binding = config->binding;
 	while (binding != NULL) {
 		if (listen(binding->socket, 16) == -1) {
-			perror("http(s) socket");
+			perror("listen(http(s))");
 			return EXIT_FAILURE;
 		}
 		binding = binding->next;
@@ -1207,7 +1251,7 @@ int run_server(t_settings *settings) {
 	binding = config->command_port;
 	while (binding != NULL) {
 		if (listen(binding->socket, 1) == -1) {
-			perror("commandchannel socket");
+			perror("listen(commandchannel)");
 			return EXIT_FAILURE;
 		}
 		binding = binding->next;
