@@ -11,51 +11,57 @@
 #include <rpcsvc/crypt.h>
 #endif
 #include "libstr.h"
+#include "liblist.h"
 #include "libmd5.h"
 #include "httpauth.h"
+#include "log.h"
 
-/* If a groupfile exist, is a user in the right group?
- *  */
-static bool group_oke(t_session *session, char *user) {
+/* If a required_group_file exist, is a user in the right group?
+ */
+bool group_oke(t_session *session, char *user, t_charlist *group) {
 	bool retval;
 	FILE *gfp;
 	char *pwfile, line[257], *item, *rest;
-	bool groupfound;
-	int i;
+	int len;
 
-	if (session->host->groupfile != NULL) {
-		retval = false;
-		pwfile = file_in_chroot(session, session->host->groupfile);
-		if ((gfp = fopen_neighbour(pwfile, "r", session->file_on_disk)) != NULL) {
-			line[256] = '\0';
-			while (fgets(line, 256, gfp) != NULL) {
-				if (split_string(line, &item, &rest, ':') == 0) {
-					groupfound = false;
-					for (i = 0; i < session->host->required_group.size; i++) {
-						if (strcmp(*(session->host->required_group.item + i), item) == 0) {
-							groupfound = true;
-							break;
-						}
-					}
-					if (groupfound) {
-						while (rest != NULL) {
-							split_string(rest, &item, &rest, ' ');
-							if (strcmp(user, item) == 0) {
-								retval = true;
-								break;
-							}
-						}
-					}
-				}
-				if (retval == true) {
-					break;
-				}
-			}
-			fclose(gfp);
-		}
-	} else {
-		retval = true;
+	if ((user == NULL) || (group->size == 0)) {
+		return true;
 	}
+	if (session->host->groupfile == NULL) {
+		return false;
+	}
+
+	pwfile = file_in_chroot(session, session->host->groupfile);
+	if ((gfp = fopen_neighbour(pwfile, "r", session->file_on_disk)) == NULL) {
+		log_file_error(session, session->host->groupfile, "groupfile not found");
+		return false;
+	}
+
+	retval = false;
+	line[256] = '\0';
+	while (fgets(line, 256, gfp) != NULL) {
+		if (split_string(line, &item, &rest, ':') == 0) {
+			if (in_charlist(item, group)) {
+				if ((len = strlen(rest)) == 0) {
+					continue;
+				}
+				if ((rest[len] == '\n') || (rest[len] == '\r')) {
+					rest[len] = '\0';
+				}
+				do {
+					split_string(rest, &item, &rest, ' ');
+					if (strcmp(user, item) == 0) {
+						retval = true;
+						break;
+					}
+				} while (rest != NULL);
+			}
+		}
+		if (retval == true) {
+			break;
+		}
+	}
+	fclose(gfp);
 
 	return retval;
 }
@@ -78,7 +84,7 @@ static char *get_password(t_session *session, char *username) {
 				if (strcmp(line, username) == 0) {
 					result = sep;
 
-					while ((*sep != '\n') && (*sep != ':') && (*sep != '\0')) {
+					while ((*sep != '\n') && (*sep != '\r') && (*sep != ':') && (*sep != '\0')) {
 						sep++;
 					}
 					if (*sep != '\0') {
@@ -87,10 +93,13 @@ static char *get_password(t_session *session, char *username) {
 					} else {
 						result = NULL;
 					}
+					break;
 				}
 			}
 		}
 		fclose(pfp);
+	} else {
+		log_file_error(session, session->host->passwordfile, "passwordfile not found");
 	}
 
 	return result;
@@ -98,7 +107,7 @@ static char *get_password(t_session *session, char *username) {
 
 /* Basic HTTP authentication.
  */
-bool basic_http_authentication(t_session *session, char *auth_str) {
+static bool basic_http_authentication(t_session *session, char *auth_str) {
 	bool retval = false;
 	char *auth_user, *auth_passwd, *passwd, *encrypted, salt[3];
 
@@ -115,7 +124,7 @@ bool basic_http_authentication(t_session *session, char *auth_str) {
 			*(auth_passwd++) = '\0';
 
 			if ((passwd = get_password(session, auth_user)) != NULL) {
-				if (group_oke(session, auth_user)) {
+				if (group_oke(session, auth_user, &(session->host->required_group))) {
 					salt[0] = *passwd;
 					salt[1] = *(passwd + 1);
 					salt[2] = '\0';
@@ -157,15 +166,37 @@ static char *unquoted(char *str) {
 
 /* Digest HTTP authentication.
  */
-bool digest_http_authentication(t_session *session, char *auth_str) {
-	bool retval = false;
+static bool digest_http_authentication(t_session *session, char *auth_str) {
+	bool retval = false, quote_found;
 	char *key, *value, *rest, *empty = "", *passwd, A1[33], A2[33], result[33];
 	char *username = empty, *realm = empty, *nonce = empty, *uri = empty, *qop = empty,
 		 *nc = empty, *cnonce = empty, *algoritm = empty, *response = empty, *opaque = empty;
 
-	key = auth_str;
-	while (key != NULL) {
-		split_string(key, &key, &rest, ',');
+	key = rest = auth_str;
+	while (*key != '\0') {
+		quote_found = false;
+		while (*rest != '\0') {
+			if (*rest == '"') {
+				if (quote_found) {
+					if (*(rest + 1) == ',') {
+						rest++;
+						*(rest++) = '\0';
+						break;
+					} else if (*(rest + 1) == '\0') {
+						rest++;
+						break;
+					} else {
+						return false;
+					}
+				}
+				quote_found = true;
+			} else if ((*rest == ',') && (quote_found == false)) {
+				*(rest++) = '\0';
+				break;
+			}
+			rest++;
+		}
+
 		if (split_string(key, &key, &value, '=') != -1) {
 			if (strcmp(key, "username") == 0) {
 				username = unquoted(value);
@@ -205,7 +236,7 @@ bool digest_http_authentication(t_session *session, char *auth_str) {
 	}
 
 	if ((passwd = get_password(session, username)) != NULL) {
-		if (group_oke(session, username)) do {
+		if (group_oke(session, username, &(session->host->required_group))) do {
 			/* Calculate A1
 			 */
 			if ((value = (char*)malloc(strlen(username) + strlen(realm) + strlen(passwd) + 3)) == NULL) {
@@ -235,7 +266,9 @@ bool digest_http_authentication(t_session *session, char *auth_str) {
 
 			/* Password match?
 			 */
-			retval = (strcmp(result, response) == 0);
+			if ((retval = (strcmp(result, response) == 0))) {
+				session->remote_user = strdup(username);
+			}
 		} while (false);
 		free(passwd);
 	}
@@ -247,15 +280,20 @@ bool digest_http_authentication(t_session *session, char *auth_str) {
  */
 bool http_authentication_oke(t_session *session) {
 	char *auth_str;
+	bool result = false;
 
 	if (session->host->passwordfile == NULL) {
 		return true;
 	} else if ((auth_str = get_headerfield("Authorization:", session->headerfields)) != NULL) {
+		auth_str = strdup(auth_str);
 		if ((strncmp(auth_str, "Basic ", 6) == 0) && (session->host->auth_method == basic)) {
-			return basic_http_authentication(session, auth_str + 6);
+			result = basic_http_authentication(session, auth_str + 6);
 		} else if ((strncmp(auth_str, "Digest ", 7) == 0) && (session->host->auth_method == digest)) {
-			return digest_http_authentication(session, auth_str + 7);
+			result = digest_http_authentication(session, auth_str + 7);
 		}
+		free(auth_str);
+
+		return result;
 	}
 
 	return false;
