@@ -298,6 +298,7 @@ static int parse_datestr(char *org_datestr, struct tm *date) {
 				}
 
 				date->tm_year -= 1900;
+				date->tm_isdst = 0;
 
 				result = 0;
 			} while (false);
@@ -361,23 +362,28 @@ FILE *fopen_neighbour(char *filename, char *mode, char *neighbour) {
 t_filelist *read_filelist(char *directory) {
 	DIR *dp;
 	t_filelist *filelist = NULL, *file;
-	char filename[257];
-	int dir_len;
+	char *filename;
+	int dir_len, stat_status;
 	struct stat status;
 	struct dirent *dir_info;
 
 	if ((dp = opendir(directory)) == NULL) {
 		return NULL;
 	}
-
-	strncpy(filename, directory, 256);
-	filename[256] = '\0';
 	dir_len = strlen(directory);
 
 	while ((dir_info = readdir(dp)) != NULL) {
 		if ((dir_info->d_name[0] != '.') || (strcmp(dir_info->d_name, "..") == 0)) {
-			strcpy(filename + dir_len, dir_info->d_name);
-			stat(filename, &status);
+			if ((filename = make_path(directory, dir_info->d_name)) == NULL) {
+				remove_filelist(filelist);
+				filelist = NULL;
+				break;
+			}
+			stat_status = stat(filename, &status);
+			free(filename);
+			if (stat_status == -1) {
+				continue;
+			}
 
 			if ((file = (t_filelist*)malloc(sizeof(t_filelist))) != NULL) {
 				if ((file->name = strdup(dir_info->d_name)) == NULL) {
