@@ -18,6 +18,7 @@ typedef struct type_filelist {
 } t_filelist;
 
 off_t filesize(char *filename);
+char *make_path(char *dir, char *file);
 enum t_fsbool contains_not_allowed_symlink(char *filename, char *webroot);
 enum t_fsbool is_directory(char *file);
 enum t_fsbool can_execute(char *file, uid_t uid, gid_t gid, t_groups *groups);
@@ -26,6 +27,6 @@ int  if_modified_since(int handle, char *datestr);
 FILE *fopen_neighbour(char *filename, char *mode, char *neighbour);
 t_filelist *read_filelist(char *directory);
 t_filelist *sort_filelist(t_filelist *filelist);
-t_filelist *remove_filelist(t_filelist *filelist);
+void remove_filelist(t_filelist *filelist);
 
 #endif
