@@ -106,6 +106,16 @@ bool in_charlist(char *item, t_charlist *list) {
 	return false;
 }
 
+void remove_charlist(t_charlist *list) {
+	if (list->size > 0) {
+		do {
+			(list->size)--;
+			sfree(*(list->item + list->size));
+		} while (list->size > 0);
+		sfree(list->item);
+	}
+}
+
 // --< accesslist >------------------------------------------------------------
 
 /* Parse an list of access levels.
@@ -266,3 +276,50 @@ t_keyvalue *remove_keyvaluelist(t_keyvalue *list) {
 
 	return NULL;
 }
+
+// --< temporary data >-------------------------------------------------------
+
+int register_tempdata(t_tempdata **tempdata, void *data, int type) {
+	t_tempdata *conf;
+
+	if (tempdata == NULL) {
+		return 1;
+	}
+
+	if ((conf = (t_tempdata*)malloc(sizeof(t_tempdata))) != NULL) {
+		conf->content = data;
+		conf->type = type;
+		conf->next = *tempdata;
+		*tempdata = conf;
+
+		return 0;
+	}
+
+	return -1;
+}
+
+void remove_tempdata(t_tempdata *tempdata) {
+	t_tempdata *conf;
+
+	while (tempdata != NULL) {
+		conf = tempdata;
+		tempdata = tempdata->next;
+
+		switch (conf->type) {
+			case tc_DATA:
+				sfree(conf->content);
+				break;
+			case tc_ACCESSLIST:
+				remove_accesslist(conf->content);
+				break;
+			case tc_KEYVALUE:
+				free(((t_keyvalue*)(conf->content))->key);
+				free(((t_keyvalue*)(conf->content))->value);
+				free(conf->content);
+				break;
+			case tc_CHARLIST:
+				remove_charlist(conf->content);
+		}
+		free(conf);
+	}
+}
