@@ -4,6 +4,11 @@
 #include <stdbool.h>
 #include "global.h"
 
+#define tc_DATA       0
+#define tc_CHARLIST   1
+#define tc_ACCESSLIST 2
+#define tc_KEYVALUE   3
+
 typedef enum { deny, allow, pwd, unknown } t_access;
 
 typedef struct type_charlist {
@@ -33,11 +38,19 @@ typedef struct type_denybotlist {
 	struct type_denybotlist *next;
 } t_denybotlist;
 
+typedef struct type_tempdata {
+	void *content;
+	int type;
+
+	struct type_tempdata *next;
+} t_tempdata;
+
 void sfree(void* ptr);
 
 void init_charlist(t_charlist *list);
 int parse_charlist(char *value, t_charlist *list);
 bool in_charlist(char *item, t_charlist *list);
+void remove_charlist(t_charlist *list);
 
 int parse_ip(char *line, t_ipv4 *l_ip);
 t_accesslist *parse_accesslist(char *line, bool pwd_allowed, t_accesslist *list);
@@ -47,4 +60,7 @@ t_access ip_allowed(t_ipv4 ip, t_accesslist *list);
 int parse_keyvaluelist(char *line, t_keyvalue **kvlist, char *seperator);
 t_keyvalue *remove_keyvaluelist(t_keyvalue *list);
 
+int register_tempdata(t_tempdata **tempdata, void *data, int type);
+void remove_tempdata(t_tempdata *tempdata);
+
 #endif
