@@ -228,7 +228,7 @@ void md5_hash(char *string, int length, char hash[33]) {
 	MD5Final(digest, &ctx);
 
 	for (i = 0; i < 16; i++) {
-		sprintf(&hash[2 * i], "%02x", digest[i]);
+		sprintf(&hash[2 * i], "%02hhx", digest[i]);
 	}
 	hash[32] = '\0';
 }
