@@ -1,3 +1,13 @@
+/*
+ * The copyright holder gives permission to link this code with the OpenSSL
+ * library and distribute linked combinations including the two. You must obey
+ * the GNU General Public License in all respects for all of the code used other
+ * than OpenSSL. If you modify this file, you may extend this exception to your
+ * version of the file, but you are not obligated to do so. If you do not wish to
+ * do so, delete this exception statement from your version.
+ *
+ */
+
 #include "config.h"
 
 #ifdef HAVE_SSL
@@ -6,6 +16,8 @@
 #include <string.h>
 #include <syslog.h>
 #include <pthread.h>
+#include <openssl/rand.h>
+#include <openssl/err.h>
 #include "alternative.h"
 #include "libssl.h"
 
@@ -80,12 +92,16 @@ static int password_callback(char *buffer, int size, int rwflag, void *data) {
 	return len;
 } 
 
-void ssl_init(void) {
+void ssl_init(char *buffer, int size) {
 	int i;
 
 	SSL_library_init();
 	SSL_load_error_strings(); 
 
+	if (buffer != NULL) {
+		RAND_seed(buffer, size);
+	}
+
 	lockcount = CRYPTO_num_locks();
 	if (lockcount > 0) {
 		if ((locks = malloc(lockcount * sizeof(pthread_mutex_t))) != NULL) {
