@@ -7,11 +7,10 @@
 #include <stdbool.h>
 #include <openssl/bio.h>
 #include <openssl/ssl.h>
-#include <openssl/err.h>
 
-void ssl_init(void);
+void ssl_init(char *buffer, int size);
 SSL_CTX *ssl_binding(char *keyfile, char *CA_cert, int verify_depth, char *dh_file, char *ciphers);
-bool ssl_accept(int socketFD, SSL **ssl, SSL_CTX *context);
+bool ssl_accept(int sock, SSL **ssl, SSL_CTX *context);
 int  ssl_receive(SSL *ssl, char *buffer, unsigned int maxlength);
 int  ssl_send(SSL *ssl, char *buffer, unsigned int length);
 int  ssl_close(SSL *ssl);
