@@ -167,41 +167,71 @@ static short hex_to_int(char c) {
 	 return -1;
 }
 
-/* Convert the special characters (%XX) to characters.
+/* Decode the URL encoded characters (%XX).
  */
-int parse_special_chars(char *line, int length) {
-	int result = length, i;
+void url_decode(char *line) {
 	short low, high;
 	char *dest = line;
 
 	if (line != NULL) {
-		for (i = 0; i < length; i++) {
+		while (*line != '\0') {
 			if (*line == '%') {
 				if ((high = hex_to_int(*(line + 1))) != -1) {
 					if ((low = hex_to_int(*(line + 2))) != -1) {
 						line += 2;
-						if ((*line = (char)(high<<4) + low) < 32) {
-							*line = '_';
-						}
-						result -= 2;
+						*line = (char)(high<<4) + low;
 					}
 				}
 			}
 			*(dest++) = *(line++);
 		}
+		*dest = *line;
 	}
+}
 
-	return result;
+void remove_dangerous_chars(char *line) {
+	short low, high;
+	char *dest = line;
+
+	if (line != NULL) {
+		while (*line != '\0') {
+			if ((*line > 0) && (*line < 32)) {
+				line++;
+				continue;
+			} else if (*line == '%') {
+				if ((high = hex_to_int(*(line + 1))) != -1) {
+					if ((low = hex_to_int(*(line + 2))) != -1) {
+						if (((high<<4) + low) < 32) {
+							line += 3;
+							continue;
+						}
+					}
+				}
+			}
+			*(dest++) = *(line++);
+		}
+		*dest = *line;
+	}
 }
 
 /* Prevent Command injection.
  */
 int prevent_CMDi(char *line) {
 	int result = 0;
+	short low, high;
+	char value;
 
 	if (line != NULL) {
 		while (*line != '\0') {
-			if ((*line == '|') || (*line == ';') || (*line == '`')) {
+			if ((value = *line) == '%') {
+				if ((high = hex_to_int(*(line + 1))) != -1) {
+					if ((low = hex_to_int(*(line + 2))) != -1) {
+						value = (char)(high<<4) + low;
+					}
+				}
+			}
+
+			if ((value == '|') || (value == ';') || (value == '`')) {
 				*line = '_';
 				result += 1;
 			}
@@ -216,7 +246,8 @@ int prevent_CMDi(char *line) {
  */
 int prevent_SQLi(char *line, int length, char **newline) {
 	int changed = 0, i, add, left, newlength = 0;
-	char *temp;
+	char *temp, value;
+	short low, high;
 
 	if ((line == NULL) || (length == 0)) {
 		return 0;
@@ -227,7 +258,17 @@ int prevent_SQLi(char *line, int length, char **newline) {
 		return -1;
 	}
 	for (i = 0; i < length; i++) {
-		if ((*(line + i) == '\'') || (*(line + i) == '`') || (*(line + i) == '\\')) {
+		value = *(line + i);
+
+		if ((value == '%') && (i + 2 < length)) {
+			if ((high = hex_to_int(*(line + i + 1))) != -1) {
+				if ((low = hex_to_int(*(line + i + 2))) != -1) {
+					value = (char)(high<<4) + low;
+				}
+			}
+		}
+
+		if ((value == '\'') || (value == '`') || (value == '\\')) {
 			*(*newline + newlength++) = '\\';
 			if (left-- == 0) {
 				add += left = 32;
@@ -256,10 +297,20 @@ int prevent_SQLi(char *line, int length, char **newline) {
  */
 int prevent_XSS(char *line) {
 	int result = 0;
+	short low, high;
+	char value;
 
 	if (line != NULL) {
 		while (*line != '\0') {
-			if ((*line == '\"') || (*line == '<') || (*line == '>') || (*line == '\'')) {
+			if ((value = *line) == '%') {
+				if ((high = hex_to_int(*(line + 1))) != -1) {
+					if ((low = hex_to_int(*(line + 2))) != -1) {
+						value = (char)(high<<4) + low;
+					}
+				}
+			}
+
+			if ((value == '\"') || (value == '<') || (value == '>') || (value == '\'')) {
 				*line = '_';
 				result += 1;
 			}
@@ -283,10 +334,13 @@ const char *http_error(int code) {
 		{102, "Processing"},
 */
 		{200, "OK"},
-/*		{201, "Created"},
+/*
+		{201, "Created"},
 		{202, "Accepted"},
 		{203, "Non-Authoritative Information"},
+*/
 		{204, "No Content"},
+/*
 		{205, "Reset Content"},
 */
 		{206, "Partial Content"},
