@@ -19,8 +19,7 @@ typedef struct type_mime {
     t_extension *extension;
 } t_mimetype;
 
-t_mimetype *read_mimetypes(char *configfile);
+int  read_mimetypes(char *configfile, t_mimetype **mime);
 char *get_mimetype(char *filename, t_mimetype *mime);
-t_mimetype *remove_mimetypes(t_mimetype *mime);
 
 #endif
