@@ -1,5 +1,6 @@
 #include "config.h"
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -49,6 +50,8 @@ static t_host *new_host(void) {
 		host->use_gz_file        = false;
 		host->require_resolve_ip = false;
 		host->access_list        = NULL;
+		host->alter_list         = NULL;
+		host->alter_fmode        = 0640;
 		host->login_message      = "Private page";
 		host->passwordfile       = NULL;
 		host->groupfile          = NULL;
@@ -56,6 +59,7 @@ static t_host *new_host(void) {
 		host->deny_bot           = NULL;
 		init_charlist(&(host->required_binding));
 		init_charlist(&(host->required_group));
+		init_charlist(&(host->alter_group));
 		host->wrap_cgi           = NULL;
 		host->groups.number      = 0;
 		host->groups.array       = NULL;
@@ -72,6 +76,7 @@ static t_host *new_host(void) {
 		host->prevent_xss        = false;
 		host->follow_symlinks    = false;
 		init_charlist(&(host->fast_cgi));
+		host->secure_url         = true;
 
 		host->next               = NULL;
 	} else {
@@ -94,12 +99,15 @@ static t_directory *new_directory(void) {
 		directory->use_gz_file_set     = false;
 		directory->follow_symlinks_set = false;
 		directory->access_list         = NULL;
+		directory->alter_list          = NULL;
+		directory->alter_fmode         = 0;
 		directory->groups.number       = 0;
 		directory->groups.array        = NULL;
 		directory->passwordfile        = NULL;
 		directory->groupfile           = NULL;
 		init_charlist(&(directory->required_group));
 		init_charlist(&(directory->image_referer));
+		init_charlist(&(directory->alter_group));
 		directory->imgref_replacement  = NULL;
 		directory->max_clients         = -1;
 		directory->nr_of_clients       = 0;
@@ -151,6 +159,7 @@ static t_binding *new_binding(void) {
 		binding->binding_id           = NULL;
 		binding->socket               = -1;
 		binding->enable_trace         = false;
+		binding->enable_trace         = false;
 		binding->max_keepalive        = 50;
 		binding->max_request_size     = 65536;
 		binding->time_for_1st_request = 5;
@@ -297,6 +306,25 @@ static bool valid_directory(char *dir) {
 	return retval;
 }
 
+static int parse_mode(char *line, mode_t *mode) {
+	mode_t mod = 0;
+	int i;
+
+	if (strlen(line) != 3) {
+		return -1;
+	}
+
+	for (i = 0; i < 3; i++) {
+		if ((line[i] < '0') || (line[i] > '9')) {
+			return -1;
+		}
+		mod = (8 * mod) + (line[i] - '0');
+	}
+	*mode = mod;
+
+	return 0;
+}
+
 static int parse_yesno(char *yesno, bool *result) {
 	if ((strcmp(yesno, "yes") == 0) || (strcmp(yesno, "true") == 0)) {
 		*result = true;
@@ -309,15 +337,17 @@ static int parse_yesno(char *yesno, bool *result) {
 	return 0;
 }
 
-int parse_passwordfile(char *line, t_auth_method *auth_method, char **pwdfile) {
-	char *file;
+int parse_credentialfiles(char *line, t_auth_method *auth_method, char **pwdfile, char **groupfile) {
+	char *file, *group;
 
-	if (*pwdfile != NULL) {
-		free(*pwdfile);
+	split_string(line, &line, &group, ',');
+	if (strcmp(line, "none") == 0) {
 		*pwdfile = NULL;
-	}
-
-	if (strcmp(line, "none") != 0) {
+	} else if (strcmp(line, "") == 0) {
+		if (group == NULL) {
+			return -1;
+		}
+	} else {
 		if (split_string(line, &line, &file, ':') == -1) {
 			return -1;
 		}
@@ -336,25 +366,10 @@ int parse_passwordfile(char *line, t_auth_method *auth_method, char **pwdfile) {
 		}
 	}
 
-	return 0;
-}
-
-int parse_requiredgroup(char *line, t_charlist *required_group, char **groupfile) {
-	char *gid;
-
-	if (*groupfile != NULL) {
-		free(*groupfile);
-		*groupfile = NULL;
-	}
-
-	if (split_string(line, &gid, &line, ':') == -1) {
-		return -1;
-	} else if (strlen(gid) >= 32) {
-		return -1;
-	} else if (parse_charlist(gid, required_group) == -1) {
-		return -1;
-	} else if ((*groupfile = strdup(line)) == NULL) {
-		return -1;
+	if (group != NULL) {
+		if ((*groupfile = strdup(group)) == NULL) {
+			return -1;
+		}
 	}
 
 	return 0;
@@ -727,7 +742,7 @@ static bool system_setting(char *key, char *value, t_config *config) {
 	return false;
 }
 
-static bool user_setting(char *key, char *value, t_host *host) {
+static bool user_setting(char *key, char *value, t_host *host, t_tempdata **tempdata) {
 	char *code;
 	int result;
 
@@ -735,6 +750,19 @@ static bool user_setting(char *key, char *value, t_host *host) {
 		if ((host->access_list = parse_accesslist(value, true, host->access_list)) != NULL) {
 			return true;
 		}
+	} else if (strcmp(key, "altergroup") == 0) {
+		if (parse_charlist(value, &(host->alter_group)) != -1) {
+			register_tempdata(tempdata, &(host->alter_group), tc_CHARLIST);
+			return true;
+		}
+	} else if (strcmp(key, "alterlist") == 0) {
+		if ((host->alter_list = parse_accesslist(value, true, host->alter_list)) != NULL) {
+			return true;
+		}
+	} else if (strcmp(key, "altermode") == 0) {
+		if (parse_mode(value, &(host->alter_fmode)) != -1) {
+			return true;
+		}
 	} else if (strcmp(key, "errorhandler") == 0) {
 		result = split_string(value, &value, &code, ',');
 		if ((strlen(value) < 128) && (*value == '/')) {
@@ -750,20 +778,29 @@ static bool user_setting(char *key, char *value, t_host *host) {
 		}
 	} else if (strcmp(key, "indexstyle") == 0) {
 		if ((host->index_style = strdup(value)) != NULL) {
+			register_tempdata(tempdata, host->index_style, tc_DATA);
 			return true;
 		}
 	} else if (strcmp(key, "loginmessage") == 0) {
 		if (strlen(value) < 64) {
 			if ((host->login_message = strdup(value)) != NULL) {
+				register_tempdata(tempdata, host->login_message, tc_DATA);
 				return true;
 			}
 		}
 	} else if (strcmp(key, "passwordfile") == 0) {
-		if (parse_passwordfile(value, &(host->auth_method), &(host->passwordfile)) == 0) {
+		if (parse_credentialfiles(value, &(host->auth_method), &(host->passwordfile), &(host->groupfile)) == 0) {
+			if (host->passwordfile != NULL) {
+				register_tempdata(tempdata, host->passwordfile, tc_DATA);
+			}
+			if (host->groupfile != NULL) {
+				register_tempdata(tempdata, host->groupfile, tc_DATA);
+			}
 			return true;
 		}
 	} else if (strcmp(key, "requiredgroup") == 0) {
-		if (parse_requiredgroup(value, &(host->required_group), &(host->groupfile)) == 0) {
+		if (parse_charlist(value, &(host->required_group)) != -1) {
+			register_tempdata(tempdata, &(host->required_group), tc_CHARLIST);
 			return true;
 		}
 #ifdef HAVE_SSL
@@ -774,6 +811,7 @@ static bool user_setting(char *key, char *value, t_host *host) {
 #endif
 	} else if (strcmp(key, "setenv") == 0) {
 		if (parse_keyvaluelist(value, &(host->envir_str), "=") != -1) {
+			register_tempdata(tempdata, host->envir_str, tc_KEYVALUE);
 			return true;
 		}
 	} else if (strcmp(key, "showindex") == 0) {
@@ -783,6 +821,7 @@ static bool user_setting(char *key, char *value, t_host *host) {
 	} else if (strcmp(key, "startfile") == 0) {
 		if (valid_file(value)) {
 			if ((host->start_file = strdup(value)) != NULL) {
+				register_tempdata(tempdata, host->start_file, tc_DATA);
 				return true;
 			}
 		}
@@ -862,10 +901,6 @@ static bool host_setting(char *key, char *value, t_host *host) {
 				}
 			}
 		}
-	} else if (strcmp(key, "indexstyle") == 0) {
-		if ((host->index_style = strdup(value)) != NULL) {
-			return true;
-		}
 	} else if (strcmp(key, "preventcmdi") == 0) {
 		if (parse_yesno(value, &(host->prevent_cmdi)) == 0) {
 			return true;
@@ -886,12 +921,10 @@ static bool host_setting(char *key, char *value, t_host *host) {
 		if (parse_yesno(value, &(host->require_resolve_ip)) == 0) {
 			return true;
 		}
-#ifdef HAVE_SSL
-	} else if (strcmp(key, "requiressl") == 0) {
-		if (parse_yesno(value, &(host->require_ssl)) == 0) {
+	} else if (strcmp(key, "secureurl") == 0) {
+		if (parse_yesno(value, &(host->secure_url)) == 0) {
 			return true;
 		}
-#endif
 	} else if (strcmp(key, "userwebsites") == 0) {
 		if (parse_yesno(value, &(host->user_websites)) == 0) {
 			return true;
@@ -924,6 +957,18 @@ static bool directory_setting(char *key, char *value, t_directory *directory) {
 		if ((directory->access_list = parse_accesslist(value, true, directory->access_list)) != NULL) {
 			return true;
 		}
+	} else if (strcmp(key, "altergroup") == 0) {
+		if (parse_charlist(value, &(directory->alter_group)) != -1) {
+			return true;
+		}
+	} else if (strcmp(key, "alterlist") == 0) {
+		if ((directory->alter_list = parse_accesslist(value, true, directory->access_list)) != NULL) {
+			return true;
+		}
+	} else if (strcmp(key, "altermode") == 0) {
+		if (parse_mode(value, &(directory->alter_fmode)) != -1) {
+			return true;
+		}
 	} else if (strcmp(key, "wrapcgi") == 0) {
 		if ((directory->wrap_cgi = strdup(value)) != NULL) {
 			return true;
@@ -947,7 +992,7 @@ static bool directory_setting(char *key, char *value, t_directory *directory) {
 			}
 		}
 	} else if (strcmp(key, "passwordfile") == 0) {
-		if (parse_passwordfile(value, &(directory->auth_method), &(directory->passwordfile)) == 0) {
+		if (parse_credentialfiles(value, &(directory->auth_method), &(directory->passwordfile), &(directory->groupfile)) == 0) {
 			return true;
 		}
 	} else if (strcmp(key, "path") == 0) {
@@ -972,7 +1017,7 @@ static bool directory_setting(char *key, char *value, t_directory *directory) {
 			}
 		}
 	} else if (strcmp(key, "requiredgroup") == 0) {
-		if (parse_requiredgroup(value, &(directory->required_group), &(directory->groupfile)) == 0) {
+		if (parse_charlist(value, &(directory->required_group)) != -1) {
 			return true;
 		}
 	} else if (strcmp(key, "setenv") == 0) {
@@ -1012,7 +1057,11 @@ static bool directory_setting(char *key, char *value, t_directory *directory) {
 static bool binding_setting(char *key, char *value, t_binding *binding) {
 	char *rest;
 
-	if (strcmp(key, "enabletrace") == 0) {
+	if (strcmp(key, "enablealter") == 0) {
+		if (parse_yesno(value, &(binding->enable_alter)) == 0) {
+			return true;
+		}
+	}else if (strcmp(key, "enabletrace") == 0) {
 		if (parse_yesno(value, &(binding->enable_trace)) == 0) {
 			return true;
 		}
@@ -1347,7 +1396,7 @@ int read_main_configfile(char *configfile, t_config *config, bool config_check)
 						if ((section == none) || (section == virtual_host)) {
 							if (host_setting(key, value, current_host)) {
 								break;
-							} else if (user_setting(key, value, current_host)) {
+							} else if (user_setting(key, value, current_host, NULL)) {
 								break;
 							}
 						} else if (section == directory) {
@@ -1399,15 +1448,21 @@ int read_main_configfile(char *configfile, t_config *config, bool config_check)
 	return retval;
 }
 
-int read_user_configfile(char *configfile, t_host *host) {
+int read_user_configfile(char *configfile, t_host *host, t_tempdata **tempdata) {
 	int  retval, counter, lines_read;
 	FILE *fp;
 	char line[MAX_LENGTH_CONFIGLINE + 1], *key, *value;
+	t_accesslist *access, *alter;
 
 	if ((fp = fopen(configfile, "r")) != NULL) {
 		line[MAX_LENGTH_CONFIGLINE] = '\0';
 		counter = retval = 0;
 
+		access = host->access_list;
+		host->access_list = NULL;
+		alter = host->alter_list;
+		host->alter_list = NULL;
+
 		while ((lines_read = fgets_multi(line, MAX_LENGTH_CONFIGLINE, fp)) != 0) {
 			if ((lines_read == -1) || (strlen(line) > MAX_LENGTH_CONFIGLINE - 1)) {
 				retval = counter + 1;
@@ -1420,7 +1475,7 @@ int read_user_configfile(char *configfile, t_host *host) {
 			if (*key != '\0') {
 				if (split_configline(key, &key, &value) != -1) {
 					strlower(key);
-					if (user_setting(key, value, host) == false) {
+					if (user_setting(key, value, host, tempdata) == false) {
 						retval = counter;
 						break;
 					}
@@ -1432,6 +1487,17 @@ int read_user_configfile(char *configfile, t_host *host) {
 		}
 		fclose(fp);
 
+		if (host->access_list == NULL) {
+			host->access_list = access;
+		} else {
+			register_tempdata(tempdata, host->access_list, tc_ACCESSLIST);
+		}
+		if (host->alter_list == NULL) {
+			host->alter_list = alter;
+		} else {
+			register_tempdata(tempdata, host->alter_list, tc_ACCESSLIST);
+		}
+
 		return retval;
 	}
 
