@@ -73,6 +73,7 @@ typedef struct type_binding {
 #endif
 
 	bool          enable_trace;
+	bool          enable_alter;
 	int           max_keepalive;
 	long          max_request_size;
 	int           time_for_1st_request;
@@ -97,11 +98,14 @@ typedef struct type_directory {
 	bool          follow_symlinks_set;
 	bool          use_gz_file;
 	bool          use_gz_file_set;
-	char          *passwordfile;
 	t_auth_method auth_method;
+	char          *passwordfile;
 	char          *groupfile;
 	t_charlist    required_group;
 	t_accesslist  *access_list;
+	t_accesslist  *alter_list;
+	t_charlist    alter_group;
+	mode_t        alter_fmode;
 	t_charlist    image_referer;
 	char          *imgref_replacement;
 	t_keyvalue    *envir_str;
@@ -140,10 +144,13 @@ typedef struct type_host {
 	t_charlist    required_binding;
 	t_denybotlist *deny_bot;
 	t_charlist    required_group;
+	t_charlist    alter_group;
 	char          *wrap_cgi;
 	t_groups      groups;
 	t_charlist    volatile_object;
 	t_accesslist  *access_list;
+	t_accesslist  *alter_list;
+	mode_t        alter_fmode;
 	t_charlist    image_referer;
 	char          *imgref_replacement;
 	t_keyvalue    *envir_str;
@@ -158,6 +165,8 @@ typedef struct type_host {
 	t_charlist    fast_cgi;
 
 	struct type_host *next;
+
+	bool          secure_url;
 } t_host;
 
 typedef struct type_config {
@@ -227,7 +236,7 @@ typedef struct type_config {
 t_config *default_config(void);
 int check_configuration(t_config *config);
 int read_main_configfile(char *configfile, t_config *config, bool config_check);
-int read_user_configfile(char *configfile, t_host *host);
+int read_user_configfile(char *configfile, t_host *host, t_tempdata **tempdata);
 t_host *get_hostrecord(t_host *host, char *hostname, t_binding *binding);
 int read_throttleconfig(char *configfile, t_throttle **throttle);
 unsigned short get_throttlespeed(char *type, t_throttle *throttle);
