@@ -32,7 +32,6 @@ static void clear_session(t_session *session) {
 	session->accept_gzip = false;
 	session->encode_gzip = false;
 	session->vars = NULL;
-	session->vars_copied = false;
 	session->http_version = NULL;
 	session->headerfields = NULL;
 	session->body = NULL;
@@ -41,7 +40,6 @@ static void clear_session(t_session *session) {
 #ifdef HAVE_SSL
 	session->cause_of_301 = missing_slash;
 #endif
-	session->body_copied = false;
 	session->header_length = 0;
 	session->content_length = 0;
 	session->file_on_disk = NULL;
@@ -57,12 +55,11 @@ static void clear_session(t_session *session) {
 	session->handling_error = false;
 	session->reason_for_403 = "";
 	session->cookie = NULL;
-	session->cookie_copied = false;
-	session->websiteroot_copied = false;
 	session->bytes_sent = 0;
 	session->output_size = 0;
 	session->return_code = 200;
 	session->error_code = -1;
+	session->tempdata = NULL;
 }
 
 /* Initialize a session-record.
@@ -109,22 +106,14 @@ void reset_session(t_session *session) {
 		}
 		pthread_mutex_unlock(&(session->directory->client_mutex));
 	}
-	if (session->vars_copied) {
-		free(session->vars);
-	}
-	if (session->body_copied) {
-		free(session->body);
-	}
-	if (session->cookie_copied) {
-		free(session->cookie);
-	}
+	remove_tempdata(session->tempdata);
 
 	/* HTTP pipelining
 	 */
 	size = session->header_length + session->content_length;
 	if ((session->bytes_in_buffer > size) && session->keep_alive) {
 		session->bytes_in_buffer -= size;
-		memcpy(session->request, session->request + size, session->bytes_in_buffer);
+		memmove(session->request, session->request + size, session->bytes_in_buffer);
 		*(session->request + session->bytes_in_buffer) = '\0';
 	} else {
 		sfree(session->request);
@@ -136,6 +125,12 @@ void reset_session(t_session *session) {
 	clear_session(session);
 }
 
+/* Free all remaining buffers
+ */
+void destroy_session(t_session *session) {
+	sfree(session->request);
+}
+
 /* Parse the HTTP headerfields from the received request.
  */
 t_headerfield *parse_headerfields(char *line) {
@@ -228,7 +223,7 @@ int get_homedir(t_session *session, char *username) {
 		memcpy(session->host->website_root, pwd->pw_dir, len);
 		*(session->host->website_root + len) = '/';
 		strcpy(session->host->website_root + len + 1, session->config->user_directory);
-		session->websiteroot_copied = true;
+		register_tempdata(&(session->tempdata), session->host->website_root, tc_DATA);
 
 		return 200;
 	}
@@ -260,11 +255,6 @@ bool duplicate_host(t_session *session) {
  */
 void restore_host(t_session *session) {
 	if (session->host_copied) {
-		if (session->websiteroot_copied) {
-			free(session->host->website_root);
-			session->websiteroot_copied = false;
-		}
-
 		free(session->host);
 		session->host_copied = false;
 	}
@@ -337,6 +327,12 @@ int copy_directory_settings(t_session *session) {
 					if (dir->access_list != NULL) {
 						session->host->access_list = dir->access_list;
 					}
+					if (dir->alter_list != NULL) {
+						session->host->alter_list = dir->alter_list;
+					}
+					if (dir->alter_fmode != 0) {
+						session->host->alter_fmode = dir->alter_fmode;
+					}
 					if (dir->image_referer.size > 0) {
 						session->host->image_referer.size = dir->image_referer.size;
 						session->host->image_referer.item = dir->image_referer.item;
@@ -345,10 +341,17 @@ int copy_directory_settings(t_session *session) {
 					if (dir->passwordfile != NULL) {
 						session->host->auth_method = dir->auth_method;
 						session->host->passwordfile = dir->passwordfile;
+						if (dir->groupfile != NULL) {
+							session->host->groupfile = dir->groupfile;
+						}
 					}
-					if (dir->groupfile != NULL) {
-						session->host->required_group = dir->required_group;
-						session->host->groupfile = dir->groupfile;
+					if (dir->required_group.size > 0) {
+						session->host->required_group.size = dir->required_group.size;
+						session->host->required_group.item = dir->required_group.item;
+					}
+					if (dir->alter_group.size > 0) {
+						session->host->alter_group.size = dir->alter_group.size;
+						session->host->alter_group.item = dir->alter_group.item;
 					}
 					break;
 				} else {
