@@ -41,8 +41,6 @@ typedef struct type_session {
 	long            header_length, content_length, buffer_size, bytes_in_buffer;
 	bool            data_sent;
 	char            *local_user;
-	bool            body_copied;
-	bool            vars_copied;
 	bool            force_quit;
 	bool            uri_is_dir;
 	bool            accept_gzip;
@@ -58,11 +56,10 @@ typedef struct type_session {
 	bool            handling_error;
 	char            *reason_for_403;
 	char            *cookie;
-	bool            cookie_copied;
-	bool            websiteroot_copied;
 	off_t           bytes_sent;
 	int             return_code;
 	int             error_code;
+	t_tempdata     *tempdata;
 
 	/* Throttling: send_buffer() in send.c
 	 */
@@ -90,6 +87,7 @@ typedef struct type_session {
 
 void init_session(t_session *session);
 void reset_session(t_session *session);
+void destroy_session(t_session *session);
 
 void set_403_reason(t_session *session, int reason);
 t_headerfield *parse_headerfields(char *line);
