@@ -14,6 +14,7 @@
 #include <time.h>
 #include <signal.h>
 #include <pthread.h>
+#include <sys/file.h>
 #include <sys/time.h>
 #include <sys/stat.h>
 #ifdef HAVE_SYS_SOCKET_H
@@ -27,6 +28,7 @@
 #include "libstr.h"
 #include "libfs.h"
 #include "target.h"
+#include "httpauth.h"
 #include "log.h"
 #include "cgi.h"
 #include "send.h"
@@ -50,6 +52,8 @@
 #define rs_DISCONNECT -2
 #define rs_FORCE_QUIT -3
 
+#define NEW_FILE -1
+
 char *chunked  = "Transfer-Encoding: chunked\r\n";
 char *hs_allow = "Allow: GET, HEAD, OPTIONS";
 char *hs_post  = ", POST";
@@ -57,6 +61,7 @@ char *hs_trace = ", TRACE";
 
 extern char *fb_filesystem;
 extern char *fb_symlink;
+extern char *fb_alterlist;
 extern char *hs_eol;
 extern char *hs_conlen;
 extern char *hs_contyp;
@@ -64,9 +69,9 @@ extern char *hs_contyp;
 /* Read a file from disk and send it to the client.
  */
 int send_file(t_session *session) {
-	char *referer, *buffer = NULL, value[VALUE_SIZE + 1], *dot, *date, *range, *range_start, *range_end;
+	char *referer, *buffer = NULL, value[VALUE_SIZE + 1], *dot, *date, *range, *range_begin, *range_end;
 	long bytes_read, total_bytes, size, speed;
-	off_t file_size, send_start, send_end, send_size;
+	off_t file_size, send_begin, send_end, send_size;
 	int  retval, handle, http_code;
 	bool invalid_referer = false, prot_oke;
 	struct stat status;
@@ -207,7 +212,7 @@ int send_file(t_session *session) {
 		log_error(session, "error while determining filesize");
 		return 500;
 	}
-	send_start = 0;
+	send_begin = 0;
 	send_end = file_size - 1;
 	send_size = file_size;
 
@@ -217,10 +222,10 @@ int send_file(t_session *session) {
 		if ((range = get_headerfield("Range:", session->headerfields)) != NULL) {
 			if (strncmp(range, "bytes=", 6) == 0) {
 				range += 6;
-				if (split_string(range, &range_start, &range_end, '-') == 0) {
+				if (split_string(range, &range_begin, &range_end, '-') == 0) {
 
-					if (*range_start != '\0') {
-						if ((send_start = str2int(range_start)) >= 0) {
+					if (*range_begin != '\0') {
+						if ((send_begin = str2int(range_begin)) >= 0) {
 							if (*range_end != '\0') {
 								if ((send_end = str2int(range_end)) >= 0) {
 									// bytes=XX-XX
@@ -232,26 +237,26 @@ int send_file(t_session *session) {
 							}
 						}
 					} else {
-						if ((send_start = str2int(range_end)) >= 0) {
+						if ((send_begin = str2int(range_end)) >= 0) {
 							// bytes=-XX
-							send_start = file_size - send_start;
+							send_begin = file_size - send_begin;
 							http_code = 206;
 						}
 					}
 
 					if (http_code == 206) {
-						if (send_start >= file_size) {
+						if (send_begin >= file_size) {
 							close(handle);
 							return 416;
 						}
-						if (send_start < 0) {
-							send_start = 0;
+						if (send_begin < 0) {
+							send_begin = 0;
 						}
 						if (send_end >= file_size) {
 							send_end = file_size - 1;
 						}
-						if (send_start <= send_end) {
-							send_size = send_end - send_start + 1;
+						if (send_begin <= send_end) {
+							send_size = send_end - send_begin + 1;
 						} else {
 							close(handle);
 							return 416;
@@ -260,14 +265,14 @@ int send_file(t_session *session) {
 
 					/* Change filepointer offset
 					 */
-					if (send_start > 0) {
-						if (lseek(handle, send_start, SEEK_SET) == -1) {
+					if (send_begin > 0) {
+						if (lseek(handle, send_begin, SEEK_SET) == -1) {
 							http_code = 200;
 						}
 					}
 
 					if (http_code == 200) {
-						send_start = 0;
+						send_begin = 0;
 						send_end = file_size - 1;
 						send_size = file_size;
 					}
@@ -303,7 +308,7 @@ int send_file(t_session *session) {
 		if (http_code == 206) {
 			if (send_buffer(session, "Content-Range: bytes ", 21) == -1) {
 				break;
-			} else if (snprintf(value, VALUE_SIZE, "%lld-%lld/%lld\r\n", (long long)send_start, (long long)send_end, file_size) == -1) {
+			} else if (snprintf(value, VALUE_SIZE, "%lld-%lld/%lld\r\n", (long long)send_begin, (long long)send_end, file_size) == -1) {
 				break;
 			} else if (send_buffer(session, value, strlen(value)) == -1) {
 				break;
@@ -347,32 +352,22 @@ int send_file(t_session *session) {
 			} else {
 				/* Normal file
 				 */
-#ifdef HAVE_SENDFILE
-				/* This is for sendfile() under Linux. It turned out that it's
-				 * slower (about 5%) then Hiawatha's 'sendfile' code.
-				 */
-				off_t offset;
+#ifdef HAVE_CACHE
+				if ((cached_object = search_cache(session, session->file_on_disk)) == NULL) {
+					cached_object = add_to_cache(session, session->file_on_disk);
+				}
+				if (cached_object != NULL) {
+					if (send_begin + send_size > cached_object->size) {
+						done_with_cached_object(cached_object, true);
+						cached_object = NULL;
+					}
+				}
 
-				if ((session->throttle == 0) 
-#ifdef HAVE_SSL
-						&& (session->binding->use_ssl == false)
-#endif
-						) {
-					send_buffer(session, NULL, 0);
-					offset = send_start;
-					if (sendfile(session->client_socket, handle, &offset, send_size) == -1) {
+				if (cached_object != NULL) {
+					if (send_buffer(session, cached_object->data + send_begin, send_size) == -1) {
 						retval = -1;
 					}
-				} else 
-#endif
-
-#ifdef HAVE_CACHE
-				if ((cached_object = search_cache(session, session->file_on_disk)) != NULL) {
-					send_buffer(session, cached_object->data, cached_object->size);
-					done_with_cached_object(cached_object);
-				} else if ((cached_object = add_to_cache(session, session->file_on_disk)) != NULL) {
-					send_buffer(session, cached_object->data, cached_object->size);
-					done_with_cached_object(cached_object);
+					done_with_cached_object(cached_object, false);
 				} else
 #endif
 				if ((buffer = (char*)malloc(FILE_BUFFER_SIZE)) != NULL) {
@@ -509,12 +504,11 @@ int execute_cgi(t_session *session) {
 						return rr_SQL_INJECTION;
 					}
 					session->vars = line;
-					session->vars_copied = true;
+					register_tempdata(&(session->tempdata), session->vars, tc_DATA);
 			}
 		}
 
 		if (session->body != NULL) {
-			session->content_length = parse_special_chars(session->body, session->content_length);
 			switch (result = prevent_SQLi(session->body, session->content_length, &line)) {
 				case -1:
 					return 500;
@@ -527,13 +521,12 @@ int execute_cgi(t_session *session) {
 					}
 					session->body = line;
 					session->content_length = result;
-					session->body_copied = true;
+					register_tempdata(&(session->tempdata), session->body, tc_DATA);
 			}
 		}
 
 		if ((session->cookie = get_headerfield("Cookie:", session->headerfields)) != NULL) {
-			parse_special_chars(session->cookie, strlen(session->cookie));
-			switch (prevent_SQLi(session->cookie, strlen(session->cookie), &line)) {
+			switch (prevent_SQLi(session->cookie, strlen(session->cookie) + 1, &line)) {
 				case -1:
 					return 500;
 				case 0:
@@ -544,7 +537,7 @@ int execute_cgi(t_session *session) {
 						return rr_SQL_INJECTION;
 					}
 					session->cookie = line;
-					session->cookie_copied = true;
+					register_tempdata(&(session->tempdata), session->cookie, tc_DATA);
 			}
 		}
 	}
@@ -757,7 +750,8 @@ int execute_cgi(t_session *session) {
 							in_body = true;
 							cgi_info.input_len = 0;
 						} else if (cgi_info.input_len > MAX_CGI_HEADER) {
-							retval = 413;
+							log_file_error(session, session->file_on_disk, "CGI's HTTP header too large");
+							retval = 500;
 							break;
 						}
 					}
@@ -777,7 +771,7 @@ int execute_cgi(t_session *session) {
 					} else if (strstr(cgi_info.input_buffer, "\r\n\r\n") == NULL) {
 						log_file_error(session, session->file_on_disk, "non-RFC HTTP header ('\\n' instead of '\\r\\n')");
 					} else {
-						log_error(session, "CGI only printed a HTTP header, no content");
+						log_file_error(session, session->file_on_disk, "CGI only printed a HTTP header, no content");
 					}
 					retval = 500;
 				}
@@ -899,23 +893,25 @@ int show_index(t_session *session) {
 
 	/* Add aliasses to directory list
 	 */
-	alias = session->host->alias;
-	while (alias != NULL) {
-		if ((file = (t_filelist*)malloc(sizeof(t_filelist))) != NULL) {
-			if ((file->name = strdup(alias->key + 1)) == NULL) {
+	if (strcmp(session->uri, "/") == 0) {
+		alias = session->host->alias;
+		while (alias != NULL) {
+			if ((file = (t_filelist*)malloc(sizeof(t_filelist))) != NULL) {
+				if ((file->name = strdup(alias->key + 1)) == NULL) {
+					remove_filelist(filelist);
+					return 500;
+				}
+				file->size = 0;
+				file->time = time(NULL);
+				file->is_dir = true;
+				file->next = filelist;
+				filelist = file;
+			} else {
 				remove_filelist(filelist);
 				return 500;
 			}
-			file->size = 0;
-			file->time = time(NULL);
-			file->is_dir = true;
-			file->next = filelist;
-			filelist = file;
-		} else {
-			remove_filelist(filelist);
-			return 500;
+			alias = alias->next;
 		}
-		alias = alias->next;
 	}
 
 	file = filelist = sort_filelist(filelist);
@@ -1201,3 +1197,165 @@ int handle_trace_request(t_session *session) {
 
 	return result;
 }
+
+static t_access allow_alter(t_session *session) {
+	char *x_forwarded_for;
+	t_ipv4 forwarded_ip;
+	t_access access;
+
+	if ((access = ip_allowed(session->ip_address, session->host->alter_list)) != allow) {
+		return access;
+	} else if ((x_forwarded_for = get_headerfield("X-Forwarded-For:", session->headerfields)) == NULL) {
+		return allow;
+	} else if (parse_ip(x_forwarded_for, &forwarded_ip) == -1) {
+		return allow;
+	} else if (ip_allowed(forwarded_ip, session->host->alter_list) == deny) {
+		return deny;
+	}
+
+	return unknown;
+}
+
+int handle_put_request(t_session *session) {
+	int handle, bytes_written, total_written = 0, result = 204;
+	char *range, *rest;
+	long write_begin, write_end, total_size;
+	off_t file_size;
+
+	switch (allow_alter(session)) {
+		case deny:
+		case unknown:
+			log_error(session, fb_alterlist);
+			return 403;
+		case allow:
+			break;
+		case pwd:
+			if (http_authentication_oke(session) == false) {
+				return 401;
+			}
+			if (group_oke(session, session->remote_user, &(session->host->alter_group)) == false) {
+				return 403;
+			}
+			break;
+	}
+
+	if (session->uri_is_dir) {
+		return 405;
+	}
+
+	range = get_headerfield("Content-Range:", session->headerfields);
+
+	if ((handle = open(session->file_on_disk, O_WRONLY)) == -1) {
+		// New file
+		if (range != NULL) {
+			return 416;
+		}
+		if ((handle = open(session->file_on_disk, O_CREAT | O_WRONLY)) == -1) {
+			log_file_error(session, session->file_on_disk, fb_filesystem);
+			return 403;
+		}
+		file_size = NEW_FILE;
+	} else {
+		// Existing file
+		if ((file_size = filesize(session->file_on_disk)) == -1) {
+			return 500;
+		}
+	}
+	if (flock(handle, LOCK_EX) == -1) {
+		log_file_error(session, session->file_on_disk, "can't lock file for writing");
+		close(handle);
+		if (file_size == NEW_FILE) {
+			unlink(session->file_on_disk);
+		}
+		return 500;
+	}
+	if (fchmod(handle, session->host->alter_fmode) == -1) {
+		log_file_error(session, session->file_on_disk, "can't change file mode");
+	}
+
+	if (range != NULL) {
+		if (strncmp(range, "bytes ", 6) != 0) {
+			result = 416;
+		} else {
+			range += 6;
+
+			if (split_string(range, &range, &rest, '-') == -1) {
+				result = 416;
+			} else if ((write_begin = str2int(range)) == -1) {
+				result = 416;
+			} else if (split_string(rest, &range, &rest, '/') == -1) {
+				result = 416;
+			} else if ((write_end = str2int(range)) == -1) {
+				result = 416;
+			} else if ((total_size = str2int(rest)) == -1) {
+				result = 416;
+			} else if (total_size != file_size) {
+				result = 416;
+			} else if (write_begin > write_end) {
+				result = 416;
+			} else if (write_begin > file_size) {
+				result = 416;
+			} else if (session->content_length != (write_end - write_begin + 1)) {
+				result = 416;
+			} else if (write_begin > 0) {
+				if (lseek(handle, write_begin, SEEK_SET) == -1) {
+					result = 500;
+				}
+			}
+		}
+	}
+
+	if (result == 204) {
+		if ((file_size != NEW_FILE) && (range == NULL)) {
+			ftruncate(handle, session->content_length);
+		}
+		while (total_written < session->content_length) {
+			if ((bytes_written = write(handle, session->body + total_written, session->content_length - total_written)) == -1) {
+				if (errno != EINTR) {
+					result = 500;
+					break;
+				}
+			} else {
+				total_written += bytes_written;
+			}
+		}
+	}
+
+	flock(handle, LOCK_UN);
+	close(handle);
+	if ((result != 204) && (file_size == NEW_FILE)) {
+		unlink(session->file_on_disk);
+	}
+
+	return result;
+}
+
+int handle_delete_request(t_session *session) {
+	switch (allow_alter(session)) {
+		case deny:
+		case unknown:
+			log_error(session, fb_alterlist);
+			return 403;
+		case allow:
+			break;
+		case pwd:
+			if (http_authentication_oke(session) == false) {
+				return 401;
+			}
+			if (group_oke(session, session->remote_user, &(session->host->alter_group)) == false) {
+				return 403;
+			}
+			break;
+	}
+
+	if (session->uri_is_dir) {
+		return 405;
+	}
+
+	if (unlink(session->file_on_disk) == -1) {
+		log_error(session, fb_filesystem);
+		return 403;
+	}
+
+	return 204;
+}
