@@ -211,7 +211,7 @@ int check_main_config(char *config_dir) {
 	int errors = 0;
 	t_line *config = NULL, *aliases = NULL, *alias, *haystack, *needles, *needle;
 	char *item, *rest, *info;
-	bool inside_section;
+	bool inside_section, has_dot;
 
 	if (quiet == false) {
 		printf("Using %s\n", config_dir);
@@ -343,8 +343,29 @@ int check_main_config(char *config_dir) {
 		haystack = haystack->next;
 	}
 
-	/* ... check
+	/* Check for dots in extensios
 	 */
+	haystack = config;
+	while (haystack != NULL) {
+		if ((strcmp(haystack->key, "cgiextension") == 0) || (strcmp(haystack->key, "extension") == 0)) {
+			has_dot = (strchr(haystack->value, '.') != NULL);
+		} else if (strcmp(haystack->key, "cgihandler") == 0) {
+			if ((rest = strchr(haystack->value, ':')) != NULL) {
+				has_dot = (strchr(rest, '.') != NULL);
+			} else {
+				has_dot = false;
+			}
+		} else {
+			has_dot = false;
+		}
+
+		if (has_dot) {
+			printf("Extensions should not contain a dot (line %d in '%s/%s')\n", haystack->linenr, config_dir, haystack->file);
+			errors++;
+		}
+
+		haystack = haystack->next;
+	}
 
 	dispose_result(config, true);
 
